/*
Freezelight Wand II v.1.0
HW - 16 LEDs on D2-D17 ("upper" on D2 and so on) placed in line, buttons on A4..A7

Печаталка для фризлайтов II v.1.0
Железо - 16 светодиодов, расставленых в линейку, на портах 2-17 (2 - "верхний" и т.д.), кнопки на A4..A7

by Artyom Chitailo (sigma72@ya.ru)
*/

#include <avr/pgmspace.h>
#include <EEPROM.h>
#include "font16.h" //файл со шрифтом - font16.cpp

//column display time, milliseconds
//время отображения каждого столбца, мс
#define COL_DELAY 10

uint16_t startcol = 0;
uint16_t stopcol = 0;

//configuration mode flag
//флаг режима настройки
bool confstate = false;

String ss0;
String ss1;
String ss2;
String ss3;

void putStr2EEPROM(String pss, byte snum){
  char buff[60];
  uint16_t staddr = (snum - 1)*64;
  for(int addr = 0; addr < 60; addr++) {
    buff[addr] = 0;
  }
  pss.toCharArray(buff,pss.length()+1);
  for(int addr = 0 ; addr < 60; addr++) {
    EEPROM.write(staddr+addr, buff[addr]);
  }
}

String getStr4EEPROM(byte snum){
  char buff[60];
  uint16_t staddr = (snum - 1)*64;
  for(int addr = 0; addr < 60; addr++) {
    buff[addr] = 0;
  }
  for(int addr = 0 ; addr < 60; addr++) {
    buff[addr] = (char)EEPROM.read(staddr+addr);
  }
  return buff;
}

//config routine
//процедура настройки
void conf() {
  byte strnum = 0;
  String nwstr = "";
  char sread;
  Serial.setTimeout(100000); //dirty but works ;)
  Serial.println("-=Freezelight Wand v.1.0=-");
  Serial.println("Strings defined: ");
  Serial.print("1: "); Serial.println(ss0);
  Serial.print("2: "); Serial.println(ss1);
  Serial.print("3: "); Serial.println(ss2);
  Serial.print("4: "); Serial.println(ss3);
  Serial.print("Which string to change? (0 - to quit config): ");
  while(Serial.available() == 0){}
  strnum = Serial.readStringUntil('\r').toInt();
   
  if(strnum == 0){
    confstate = false;
    Serial.println("Bye!");
    return;
  }
  if(strnum > 4) {
  Serial.println("Invalid number!");
  Serial.println();
  delay(1000);
  return;
  }
  
  Serial.println();
  Serial.print("New string (press Enter to keep old): ");
 
  while(Serial.available() == 0){}
    nwstr = Serial.readStringUntil('\r');
     Serial.println(nwstr);
    if(nwstr.length() == 0) {
      return;
    }
     
    putStr2EEPROM(nwstr, strnum);
    Serial.println("Stored! ");
    delay(500);
    Serial.println();
    loadstrs();
    return;
}
    
void loadstrs() {
  ss0 = getStr4EEPROM(1);
  ss1 = getStr4EEPROM(2);
  ss2 = getStr4EEPROM(3);
  ss3 = getStr4EEPROM(4);
}
  
void setup() {
  byte p;
  pinMode(A4, INPUT_PULLUP);
  pinMode(A5, INPUT_PULLUP);

  for(p = 2; p < 18; p++) {
    pinMode(p, OUTPUT); }

  for(p = 2; p < 18; p++) {
    digitalWrite(p, HIGH);
    delay(50);
  }
  for(p = 2; p < 18; p++) {
    digitalWrite(p, LOW);
    delay(50);
  }
  //load previously define string
  //загрузить сохраненные строки
  loadstrs();
  
  Serial.begin(9600);
  
  if(digitalRead(A4) == LOW) { //go to config - переход на конфиг
    confstate = true;
    }
  }

void showchar() {
//read the array and put it to the LED line
//читаем из массива и выводим на линейку
  for(int a = startcol; a < stopcol + 1; a++) {
    for(int c=0; c < 16; c++){
    digitalWrite(c + 2, bitRead(pgm_read_word(&(stripe[a])),15 - c));
    }
    delay(COL_DELAY);
  }
}

//split string into chars and find start and end columns using look-up table
//нарезаем строку на символы и опрелеяем по таблице начальный и конечный столбец
void dispstripe(byte strslot) {
String curstr;
if(strslot == 0){curstr = ss0;}
if(strslot == 1){curstr = ss1;}
if(strslot == 2){curstr = ss2;}
if(strslot == 3){curstr = ss3;}

for(int r = 0; r < curstr.length(); r++){
  byte curchar = curstr.charAt(r);
  startcol = pgm_read_word(&(lktab[curchar][0]));
  stopcol  = pgm_read_word(&(lktab[curchar][1]));
  showchar();  
}
}

void loop() {
  if(confstate) {conf();}
//wait for pressed button to run
//ждем нажатия на кнопки для запуска строки
   if(digitalRead(A4) == LOW) {dispstripe(0);}
   if(digitalRead(A5) == LOW) {dispstripe(1);}
   if(analogRead(A6) < 100) {dispstripe(2);} //A6 and A7 not digital inputs but ADC only
   if(analogRead(A7) < 100) {dispstripe(3);} //A6 и A7 - не цифровые входы, а входы АЦП
  delay(50);
}
